/*
    Sets the frequency of the TV tuner.
    Copyright (C) 2006  Andrew Miller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define __user
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/videodev2.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

int
main(int argc, char** argv)
{
	struct v4l2_frequency f_set, f_get;
	struct v4l2_control ctrl;
	struct v4l2_capability cap;
	double start_freq, end_freq, freq_stepsize, freq, subcarrier;
	int fd;
	u_int32_t freqmaj, freqmin, soundfreq, start_subcarrier, end_subcarrier, subcarrier_stepsize;

	if (argc < 7)
	{
		printf("Usage: scanfreq <start freq (MHz)> <end freq (MHz)> <freq stepsize> <start subcarrier> <end subcarrier> <subcarrier stepsize>\n");
		return -1;
	}
		
	start_freq = strtod(argv[1], NULL);
	end_freq = strtod(argv[2], NULL);
	freq_stepsize = strtod(argv[3], NULL);
	start_subcarrier = strtoul(argv[4], NULL, 10);
	end_subcarrier = strtoul(argv[5], NULL, 10);
	subcarrier_stepsize = strtoul(argv[6], NULL, 10);

	if (start_freq < 44.0 || start_freq > 958.0)
	{
		printf("Sorry, start frequency not supported by hardware :(\n");
		return 0;
	}
	if (end_freq < 44.0 || end_freq > 958.0)
	{
		printf("Sorry, end frequency not supported by hardware :(\n");
		return 0;
	}
	if(end_freq <= start_freq)
	{
		printf("End frequency must be strictly larger than start frequency");
		return 0;
	}
	if(freq_stepsize <= 0) {
		printf("Stepsize must be positive");
		return 0;
	}

	fd = open("/dev/video1", O_RDWR);
	if (fd == -1)
	{
		perror("open radio");
		return -1;
	}

	ioctl(fd, VIDIOC_QUERYCAP, &cap);
	printf("Device capability flags: %u\n", cap.capabilities);
	ctrl.id = V4L2_CID_AUDIO_MUTE;
	ctrl.value = 0;
	ioctl(fd, VIDIOC_S_CTRL, &ctrl);

	for(soundfreq=start_subcarrier;soundfreq<=end_subcarrier;soundfreq+=subcarrier_stepsize)
	{
		for(freq = start_freq;freq<=end_freq;freq+=freq_stepsize)
		{		
			/* Make the main tuner do most of the work. */
			freqmaj = (u_int32_t)(ceil(16.0 * freq));
			
			/* Now compute the minor carrier to pick up... */
			subcarrier = ((freqmaj / 16.0) - freq) * 1000;
			while (subcarrier < soundfreq)
			{
				freqmaj--;
				subcarrier += 62.5;
			}
			freqmin = (u_int32_t)subcarrier;
			printf("Main frequency set to %.4f MHz, then taking carrier at %d kHz\n",(freqmaj / 16.0), freqmin);

			f_set.tuner = 0;
			f_set.type = V4L2_TUNER_ANALOG_TV;
			f_set.frequency = freqmaj;
			f_set.reserved[0] = freqmin;
			//f_set.reserved[1] = audio_mode; // Audio mode : see saa7134.h for enum-mapping
			ioctl(fd, VIDIOC_S_FREQUENCY, &f_set);

			ioctl(fd, VIDIOC_G_FREQUENCY, &f_get);
			printf("Frequency is currently %.4f MHz and subcarrier is %d kHz\n",(f_get.frequency/16.0),f_get.reserved[0]);
			sleep(1);
		}
	}
	close(fd);
	return 0;
}
