/*
    Sets the frequency of the TV tuner.
    Copyright (C) 2006  Andrew Miller

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define __user
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/videodev2.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

int
main(int argc, char** argv)
{
  struct v4l2_frequency f_set, f_get;
  struct v4l2_control ctrl;
  struct v4l2_capability cap;
  double reqf, carrier;
  int fd, returnCode;
  u_int32_t freqmaj, freqmin, soundfreq, audio_mode;

  if (argc < 3)
  {
    printf("Usage: setfreq frequency soundfreq\n");
    return -1;
  }
	if(argc < 4)
		audio_mode = 1;
	else
		audio_mode = strtoul(argv[5], NULL, 10);

  reqf = strtod(argv[1], NULL);
  if (reqf < 44.0 || reqf > 958.0)
  {
    printf("Sorry, frequency not supported by hardware :(\n");
    return 0;
  }

  soundfreq = strtoul(argv[2], NULL, 10);

  /* Make the main tuner do most of the work. */
  freqmaj = (u_int32_t)(ceil(16.0 * reqf));

  /* Now compute the minor carrier to pick up... */
  carrier = ((freqmaj / 16.0) - reqf) * 1000;
  while (carrier < soundfreq)
  {
    freqmaj--;
    carrier += 62.5;
  }
  freqmin = (u_int32_t)soundfreq;

  printf("Main frequency set to %.4f MHz, then taking carrier at %d kHz\n",(freqmaj / 16.0), freqmin);

  fd = open("/dev/video1", O_RDWR);
  if (fd == -1)
  {
    perror("open radio");
    return -1;
  }

  ioctl(fd, VIDIOC_QUERYCAP, &cap);
  printf("Device capability flags: %x\n", cap.capabilities);

  ctrl.id = V4L2_CID_AUDIO_MUTE;
  ctrl.value = 0;
  returnCode = ioctl(fd, VIDIOC_S_CTRL, &ctrl);
  printf("VIDIOC_S_CTRL returned with code %d\n",returnCode);

  f_set.tuner = 0;
  f_set.type = V4L2_TUNER_ANALOG_TV;
  f_set.frequency = freqmaj;
  f_set.reserved[0] = freqmin;
  //f_set.reserved[1] = audio_mode; // Audio mode : see saa7134.h for enum-mapping
  returnCode = ioctl(fd, VIDIOC_S_FREQUENCY, &f_set);
  printf("VIDIOC_S_FREQUENCY returned with code %d\n",returnCode);

  returnCode = ioctl(fd, VIDIOC_G_FREQUENCY, &f_get);
  printf("VIDIOC_G_FREQUENCY returned with code %d\n",returnCode);
  printf("Frequency is currently %.4f MHz and subcarrier is %d kHz\n",(f_get.frequency/16.0),f_get.reserved[0]);

  close(fd);
  return 0;
}
